import asyncio
import concurrent.futures
import functools
import time
from pathlib import Path
from urllib.parse import urlparse

import requests
import os

URL = input("Введите ссылку для загрузки: ")


async def get_size(url):
    response = requests.head(url, allow_redirects=True)
    if 200 <= response.status_code <= 299:
        size = int(response.headers['Content-Length'])
        name = response.headers.get('filename')
        if not name:
            name = Path(urlparse(response.url).path).name
        return size, name


def download_range(url, start, end, output):
    headers = {'Range': f'bytes={start}-{end}'}
    response = requests.get(url, headers=headers, allow_redirects=True)

    with open(output, 'wb') as f:
        for part in response.iter_content(1024):
            f.write(part)


async def download(run, loop, url):
    file_size, name = await get_size(url)
    chunk_size = int(file_size / 3)
    chunks = range(0, file_size, chunk_size)

    tasks = [
        run(
            download_range,
            url,
            start,
            start + chunk_size - 1,
            f'{name}.part{i}',
        )
        for i, start in enumerate(chunks)
    ]

    await asyncio.wait(tasks)

    with open(name, 'wb') as o:
        for i in range(len(chunks)):
            chunk_path = f'{name}.part{i}'

            with open(chunk_path, 'rb') as s:
                o.write(s.read())

            os.remove(chunk_path)


if __name__ == '__main__':
    t = time.time()
    executor = concurrent.futures.ThreadPoolExecutor(max_workers=3)
    loop = asyncio.new_event_loop()
    run = functools.partial(loop.run_in_executor, executor)

    asyncio.set_event_loop(loop)

    try:
        loop.run_until_complete(
            download(run, loop, URL)
        )
    finally:
        loop.close()
        print(f'Время загрузки файла частями асинхронной функции: {time.time() - t:.2f}')
